import * as tslib_1 from "tslib";
/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
import { Injectable } from '@angular/core';
/**
 * Class to abstract get and set values from an object or an array
 * using dot-notation string as the path to the property
 *
 */
var NrfModelSetterService = /** @class */ (function () {
    function NrfModelSetterService() {
        var _this = this;
        /**
         * Get the final property of the dot notation path.
         * This will be the property that will receive the new value
         */
        this.getTargetPropToSet = function (obj, key, i, pathPieces) {
            var /** @type {?} */ nextKey = pathPieces[i + 1];
            var /** @type {?} */ isLast = nextKey == null;
            nextKey = nextKey || pathPieces['finalKey'];
            var /** @type {?} */ isArrayKey = _this.isArrayKey(nextKey);
            var /** @type {?} */ prop = obj[key];
            if (!prop) {
                prop = isArrayKey ? [] : {};
                obj[key] = prop;
            }
            else if (isLast) {
                prop = isArrayKey ? tslib_1.__spread(prop) : Object.assign({}, prop);
                obj[key] = prop;
            }
            return prop;
        };
    }
    /**
     * Get a value using a dot-notation string to find the nested property
     *
     * @param {?} path Dot notation of the value to be get
     * @param {?} model An object or an Array
     * @param {?=} separator An optional param to specify a different separator, default '.'
     *
     * ```typescript
     * const nrfEntity = {
     *    user: {
     *        name: 'John Doe'
     *    }
     * };
     * nestedProps.getValue('user.name', nrfEntity);
     * // => 'John Doe'
     *
     * nestedProps.getValue('user.email', nrfEntity);
     * // => null
     * ```
     * @return {?}
     */
    NrfModelSetterService.prototype.getValue = function (path, model, separator) {
        if (separator === void 0) { separator = '.'; }
        if (path == null || !model) {
            return null;
        }
        try {
            var /** @type {?} */ pathPieces = this.generatePathPieces(path, separator);
            return pathPieces.reduce(this.piecesReducer, model) || null;
        }
        catch (err) {
            return null;
        }
    };
    /**
     * @ignore
     *
     * Return the value associated to the given key.
     *
     * If the key is empty, the last element of an array is returned
     * @param {?} obj
     * @param {?} key
     * @return {?}
     */
    NrfModelSetterService.prototype.piecesReducer = function (obj, key) {
        return key === '' ? obj.slice(-1)[0] : obj[key];
    };
    /**
     * Set a value using a dot-notation string.
     *
     * If the property does not exists, the entire path will be created.
     *
     * @param {?} path Dot notation of the value to be get
     * @param {?} value The value to be set
     * @param {?} model An object or an Array
     * @param {?=} separator An optional param to specify a different separator, default '.'
     *
     * ```typescript
     * nestedProps.setValue('user.age', 35, { user: { name: 'John Doe' } });
     * ```
     * @return {?}
     */
    NrfModelSetterService.prototype.setValue = function (path, value, model, separator) {
        if (separator === void 0) { separator = '.'; }
        if (!(path && model)) {
            return null;
        }
        try {
            var /** @type {?} */ pathPieces = this.generatePathPieces(path, separator);
            var /** @type {?} */ finalKey = pathPieces.pop();
            ((pathPieces))['finalKey'] = finalKey;
            var /** @type {?} */ targetProp = pathPieces.reduce(this.getTargetPropToSet, model) || null;
            if (finalKey === '') {
                targetProp.push(value);
            }
            else {
                targetProp[finalKey] = value;
            }
        }
        catch (err) {
            console.error('NestedProps setValue error: ', err);
        }
    };
    /**
     * @ignore
     * Internal function to define if a given key is for an array or an object
     * @param {?} key
     * @return {?}
     */
    NrfModelSetterService.prototype.isArrayKey = function (key) {
        return key === '' || !isNaN(parseInt(key, 10));
    };
    /**
     * Generate the dot-notation path as an Array to iterate over.
     * @param {?} path
     * @param {?} separator
     * @return {?}
     */
    NrfModelSetterService.prototype.generatePathPieces = function (path, separator) {
        return path
            .replace('[', separator)
            .replace(']', '')
            .split(separator);
    };
    return NrfModelSetterService;
}());
export { NrfModelSetterService };
NrfModelSetterService.decorators = [
    { type: Injectable },
];
function NrfModelSetterService_tsickle_Closure_declarations() {
    /** @type {!Array<{type: !Function, args: (undefined|!Array<?>)}>} */
    NrfModelSetterService.decorators;
    /**
     * @nocollapse
     * @type {function(): !Array<(null|{type: ?, decorators: (undefined|!Array<{type: !Function, args: (undefined|!Array<?>)}>)})>}
     */
    NrfModelSetterService.ctorParameters;
    /**
     * Get the final property of the dot notation path.
     * This will be the property that will receive the new value
     * @type {?}
     */
    NrfModelSetterService.prototype.getTargetPropToSet;
}
//# sourceMappingURL=model-setter.service.js.map
